"""TexSoup's main utility is the ``TexSoup`` function.

Invoke this function on a LaTeX string or file handler to obtain a parse
tree with navigation, search, and modification utilities.
"""

from .tex import read
from .data import TexNode

__version__ = '0.3.1'


# noinspection PyPep8Naming
def TexSoup(tex_code, skip_envs=(), tolerance=0):
    r"""
    At a high-level, parses provided Tex into a navigable, searchable
    structure. This is accomplished in two steps:

    1. Tex is parsed, cleaned, and packaged.
    2. Structure fed to TexNodes for a searchable, coder-friendly interface.

    :param Union[str,iterable] tex_code: the Tex source
    :param Union[str] skip_envs: names of environments to skip parsing
    :param int tolerance: error tolerance level (only supports 0 or 1)
    :return: :class:`TexSoup.data.TexNode` object representing tex document

    >>> from TexSoup import TexSoup
    >>> soup = TexSoup(r'''
    ... \begin{document}
    ...
    ... \section{Hello \textit{world}.}
    ...
    ... \subsection{Watermelon}
    ...
    ... (n.) A sacred fruit. Also known as:
    ...
    ... \begin{itemize}
    ... \item red lemon
    ... \item life
    ... \end{itemize}
    ...
    ... Here is the prevalence of each synonym.
    ...
    ... \begin{tabular}{c c}
    ... red lemon & uncommon \\ \n
    ... life & common
    ... \end{tabular}
    ...
    ... \end{document}
    ... ''')
    >>> soup.section
    \section{Hello \textit{world}.}
    >>> soup.section.name
    'section'
    >>> soup.section.string
    'Hello \\textit{world}.'
    >>> soup.section.parent.name
    'document'
    >>> soup.tabular
    \begin{tabular}{c c}
    red lemon & uncommon \\ \n
    life & common
    \end{tabular}
    >>> soup.tabular.args[0].string
    'c c'
    >>> soup.itemize
    \begin{itemize}
    \item red lemon
    \item life
    \end{itemize}
    >>> soup.item
    \item red lemon
    <BLANKLINE>
    >>> list(soup.find_all('item'))
    [\item red lemon
    , \item life
    ]
    >>> soup = TexSoup(r'''\textbf{'Hello'}\textit{'Y'}O\textit{'U'}''')
    >>> soup.textbf.delete()
    >>> 'Hello' not in repr(soup)
    True
    >>> soup.textit.replace_with('S')
    >>> soup.textit.replace_with('U', 'P')
    >>> soup
    SOUP
    """
    parsed, src = read(tex_code, skip_envs=skip_envs, tolerance=tolerance)
    return TexNode(parsed, src=src)



#!/usr/bin/env python3
'''
 Name : Elowan
 Creation : 24-06-2023 20:53:52
 Last modified : 28-06-2023 16:44:30
'''
from .TexSoup import TexSoup, utils, data
import re
import os

from .CONST import disponible_environments, skip_envs
from .latex2png import pnglatex

# Soup = l'intérieur d'une balise

class Converter:
    """
    Classe de conversion d'un fichier latex en markdown
    """

    def __init__(self, filename):
        """
        Initialise le convertisseur

        :param filename: le nom du fichier à convertir
        :type filename: str
        """
        self.filename = filename.split("/")[-1]
        self.sourcedir = "/".join(filename.split("/")[:-1]) + "/"
        self.workingdir = os.getcwd() + "/"
        self.imgdir = self.workingdir + "images/{}/".format(
            filename.split("/")[-1].split('.')[0].replace(' ', '_'))
        self.stringSoup = ""
        self.packages = []

    def frameAnalyse(self, soup):
        """
        Analyse le contenu d'un frame et le transforme en markdown

        (Prend en compte les frames fragiles et les titres pas dans des 
        frametitle)

        :param soup: le contenu du frame
        :type soup: TexSoup.TexNode
        :return: le contenu du frame en markdown
        """
        # Si la frame est sans paramètres
        if len(soup.args) == 0:
            return self.transformSoupToMarkdown(list(soup.contents))
        
        # Si la frame est fragile
        if soup.args[0] == '[fragile]':
            string = ''

            # Si la frame a un titre
            if len(soup.args) >= 2:
                string += "\n#### " + Converter.removeWeirdSpacing(str(soup.args[1])) \
                        .replace('{', '').replace('}', '') + "\n\n"
                return string + self.transformSoupToMarkdown(list(soup.contents)[2:])
            
            else:
                return "\n" + self.transformSoupToMarkdown(list(soup.contents)[1:])
        else:
            # Si la frame a un titre mais pas de mention fragile
            string = "\n#### " + Converter.removeWeirdSpacing(str(soup.args[0])).replace('{', '').replace('}', '') + "\n\n"
            return string + self.transformSoupToMarkdown(list(soup.contents)[1:])
        
    @staticmethod
    def getEnvironment(string):
        """
        Récupère le contenu d'un environnement dans un string

        Trouve le premier '{' et renvoie tout ce qu'il y a dedans 

        :param string: le string à analyser
        :type string: str
        :return: le contenu de l'environnement
        """
        count = None
        ignoreLine = False
        for i in range(len(string)):
            if ignoreLine:
                continue

            if string[i] == '%':
                ignoreLine = True
                continue

            if string[i] == '{':
                if count == None: count = 0
                count += 1
            elif string[i] == '}':
                count -= 1
            if count == 0:
                return string[string.find('{')+1:i]

    @staticmethod
    def getWeirdEnvironment(string):
        """
        Récupère le contenu d'une commande de la forme "\env ... {}" 
        dans un string avec ... les options de la commande

        :param string: le string à analyser
        :type string: str
        :return: (
            la commande, 
            le contenu de l'environnement, 
            la commande complète
        )
        """
        command = ""

        # Récupère la commande latex
        for i in range(len(string)):
            if string[i] == '\\':
                command += string[i:string.find('{')]
                break

        # Récupère le contenu de l'environnement
        envContent = Converter.getEnvironment(string[len(command):])
        
        return string[string.find('\\'):len(command)+len(envContent)+3]

    def renderImage(self, name, string):
        """
        Rend une image latex en png

        :param name: le nom de l'image
        :type name: str
        :param string: le contenu de l'image
        :type string: str
        :return: le nom de l'image
        :rtype: str
        """
        print("/!\ Commande impossible à parser : {}".format(
                    Converter.removeWeirdSpacing(string)))
        print("Conversion en image...\n")
        
        imageNumber = len([name for name in os.listdir(self.imgdir) if os.path.isfile(os.path.join(self.imgdir, name))])

        # Convertit le latex en image
        try:
            filename = "{}_{}.png".format(name, imageNumber)
            pnglatex(string, self.imgdir + "{}".format(filename), packages=self.packages)

        except ValueError:
            print("/!\ Erreur lors de la conversion en image\n")
            return "\n!!!danger \"Erreur de convertion : {}\"\n".format(
                    Converter.removeWeirdSpacing(filename))
        

        else:
            print("Conversion en image réussie\n")
            return "\n\n![{}]({})\n\n".format(name, self.imgdir + "{}".format(filename))

    # Fonctions de transformation de balise en markdown
    def get_functions_for_envs(self, fun):
        functions_for_envs = {
            # MetaBalises
            "input": lambda x, indent: "\n\n!!!danger \"Include\"\n", # TODO: include the file
            "frame": lambda x, indent: self.frameAnalyse(x),
            "title": lambda x, indent: "# " + Converter.removeWeirdSpacing(str(x.string)) + "\n\n",
            "section": lambda x, indent: "\n## " + Converter.removeWeirdSpacing(str(x.string)) + "\n\n",
            "subsection": lambda x, indent: "\n### " + Converter.removeWeirdSpacing(str(x.string)) + "\n\n",
            "frametitle": lambda x, indent: "\n\n#### " + Converter.removeWeirdSpacing(str(x.string)) + "\n\n",
            "framesubtitle": lambda x, indent: "\n##### " + Converter.removeWeirdSpacing(str(x.string)) + "\n\n",

            
            # Blocs à part entière
            "label": lambda x, indent: "", # On dégage les labels
            "tiny": lambda x, indent: str(self.transformSoupToMarkdown(list(x.contents))),
            "small": lambda x, indent: str(self.transformSoupToMarkdown(list(x.contents))),
            "ref": lambda x, indent: "[" + Converter.removeWeirdSpacing(str(x.string)) + "](#"
                                    + str(x.string) + ")",
            "tabular": lambda x, indent: self.renderImage(str(x.name), str(x)),

            "figure": lambda x, indent: "\n" + self.renderImage(x.name, str(x)),
            
            "itemize": lambda x, indent: "\n\n" + "\n".join([indent + "- " + str(
                                            self.transformSoupToMarkdown(list(elmt))
                                            ).rstrip().lstrip() 
                                            for elmt in list(x.children)]) + "\n\n",

            "enumerate": lambda x, indent: "\n\n" + "\n".join([indent + "1. " + str(
                                            self.transformSoupToMarkdown(list(elmt), indent)
                                            ).rstrip().lstrip()
                                            for elmt in list(x.children)]) + "\n\n",

            "lstlisting": lambda x, indent: "\n```{} linenums=\"1\"\n".format(str(x.contents[0])
                                            .replace("style=", "")) + 
                                            str(x.contents[1]).replace('    ', ' ') + 
                                            "\n```\n",

            "columns": lambda x, indent: str(self.transformSoupToMarkdown(list(x.contents[2:]), indent)),
            "column": lambda x, indent: str(self.transformSoupToMarkdown(list(x.contents[1:]), indent)),

            "defn": lambda x, indent: '\n\n!!!quote "Définition"\n' + "     " + 
                                            str(self.transformSoupToMarkdown(list(x.contents), 
                                            indent + "     ")) + "\n\n",

            "rem": lambda x, indent: '\n\n!!!tip ""\n    **Remarque**\n\n' + "     " +
                                            str(self.transformSoupToMarkdown(list(x.contents), 
                                            indent + "     ")) + "\n\n",

            "prop": lambda x, indent: '\n\n!!!warning ""\n    **Propriété**\n\n' + "     " +
                                            str(self.transformSoupToMarkdown(list(x.contents), 
                                            indent + "     ")) + "\n\n",
            "block": lambda x, indent: "\n\n" + str(self.transformSoupToMarkdown(list(x.contents),
                                            indent)) + "\n\n",
            "cod": lambda x, indent: "\n`" + str(self.transformSoupToMarkdown(list(x.contents),
                                            indent)) + "`\n",
            

            # Inline elements
            "href": lambda x, indent: "[" + Converter.removeWeirdSpacing(str(x.args[1].string)) + 
                                            "](" + str(x.args[0].string) + ")",

            "emph": lambda x, indent: " *" + str(self.transformSoupToMarkdown(list(x.contents)
                                            )) + "*",
            "enquote": lambda x, indent: "\"" + str(self.transformSoupToMarkdown(list(x.contents)
                                            )) + "\"",
            "textcolor": lambda x, indent: "<span style='color:{}'>".format(
                                            str(x.args[0].string)
                                            ) + str(self.transformSoupToMarkdown(
                                                list(x.contents[1:]))
                                            ) + "</span>",

            "displaymath": lambda x, indent: "\n\n$$\n" + " ".join(
                                            [ str(x).rstrip().lstrip() for x in x.contents]
                                            ) + "$$\n\n",
            "BraceGroup": lambda x, indent: str(self.transformSoupToMarkdown(
                                            list(x.contents), indent)),

            # "alert": lambda x, indent: str(self.transformSoupToMarkdown(
            #                                 list(TexSoup(getEnvironment(self.stringSoup[x.position:]), 
            #                                 skip_envs=skip_envs)), indent)),

            "textbf": lambda x, indent: "**" + str(self.transformSoupToMarkdown(
                                            list(x.contents))
                                            ) + "**",
            
            "gbox": lambda x, indent: " `" + str(self.transformSoupToMarkdown(
                                            list(x.contents))
                                            ) + "`",
            "textwidth": lambda x, indent: str(self.transformSoupToMarkdown(
                                            list(x.contents))
                                            ),

            # Abréviations (TODO: à vérifier)
            "node": lambda x, indent: "noeud",
            "ie": lambda x, indent: "ie",
            "ssi": lambda x, indent: "si et seulement si",
            "": lambda x, indent: self.transformSoupToMarkdown(list(x.contents), indent),
        }

        if fun in functions_for_envs:
            return functions_for_envs[fun]
        
        else:
            return None

    def transformSoupToMarkdown(self, soupList, indent=""):
        """
        Transforme une liste de soup en markdown, si un % est placé au début de la ligne,
        alors la ligne n'est pas gardée. La fonction est récursive.

        :param list: une liste de soup
        :type list: List

        :return: string
        """
        resultString = ''
        toSkip = -1

        # Parcours de la liste de soup
        for i in range(len(soupList)):
            soup = soupList[i]

            if soup.position < toSkip:
                continue

            # Si c'est un commentaire, on passe à la ligne suivante
            if str(soup).startswith('%'):
                continue

            # Si c'est un simple texte ou des maths, on le retourne
            if type(soup) == utils.Token or soup.name == "$":
                resultString += ' ' + Converter.removeWeirdSpacing(str(soup))

                if type(soup) != data.TexNode:
                    resultString = resultString.rstrip().lstrip()
                    
                continue

            # Si on a prévu une fonction pour cette balise, on l'applique
            func = self.get_functions_for_envs(soup.name)
            if func is not None:
                resultString += str(func(soup, indent))
                continue
            
            # Si la balise n'est pas prise en charge par le script
            # qui se chargera de l'afficher sur le site, on affiche un warning
            if soup.name not in disponible_environments:
                string = Converter.getWeirdEnvironment(self.stringSoup[soup.position-1:])

                # Transforme la balise en image             
                string = self.renderImage(soup.name, string)

                toSkip = soup.position + len(string) + 1
                
                resultString += string

                # resultString += "\n\n$$\n" + Converter.removeWeirdSpacing(str(soup)).rstrip().lstrip() + "\n$$\n\n"
                continue

            # Sinon on applique la fonction sur tous les enfants de la balise
            resultString += self.transformSoupToMarkdown(list(soup.contents), indent)

            resultString += "\n"
            
        return resultString

    @staticmethod
    def removeWeirdSpacing(string):
        """
        Enlève les espaces/saut à la ligne en trop dans une string

        :param string: la string à modifier
        :type string: string

        :return: string
        """
        # On enlève les sauts à la ligne, les espaces et les % en trop
        reg1 = re.compile(r'\s+')
        reg2 = re.compile(r'[\n\r%]')
        string = re.sub(reg1, ' ', string)
        return re.sub(reg2, '', string)

    def includeFiles(self, soup):
        """
        Include les fichiers .tex dans le fichier principal

        :param soup: le fichier principal
        :type soup: TexSoup

        :return: string
        """
        # Récupère les fichiers à inclure
        inputs = soup.find_all('input')
        for input_soup in inputs:
            # Ouvre le fichier
            try:
                with open(self.sourcedir + str(input_soup.args[0].string), 'r') as f:
                    stringSoup = f.read()

                # Ajoute le contenu du fichier au fichier principal
                input_soup.replace_with(TexSoup(stringSoup, skip_envs=skip_envs))
            
            except FileNotFoundError:
                print("/!\ Impossible d'importer le fichier " + 
                    str(input_soup.args[0].string))
                
        return TexSoup(str(soup), skip_envs=skip_envs)

    @staticmethod
    def getAllImportedPackages(soup):
        """
        Récupère tous les packages importés dans le fichier

        :param soup: le fichier principal
        :type soup: TexSoup

        :return: List
        """
        packages = []
        for package in soup.find_all('usepackage'):
            packages.append(str(package))
        return packages

    @staticmethod
    def removeComments(soup):
        """
        Enlève les commentaires du fichier

        :param soup: le fichier principal
        :type soup: TexSoup

        :return: string
        """
        # Récupère tous les commentaires
        ignoreLine = False
        soupstr = str(soup)
        outstring = ""
        for i in range(len(soupstr)):
            if soupstr[i] == '%':
                ignoreLine = True
                continue

            if soupstr[i] == '\n':
                ignoreLine = False
                
            if not ignoreLine:
                outstring += soupstr[i]

        return TexSoup(outstring)

    @staticmethod
    def removeOldImages(imgdir):
        """
        Supprime les images précédentes
        """
        if os.path.exists(imgdir):
            for file in os.listdir(imgdir):
                os.remove(imgdir + file)

    def run(self):
        """
        Fonction principale du script, transforme un fichier .tex en markdown
        """
        # Ouverture du fichier
        with open(self.sourcedir + self.filename, 'r') as f:
            stringSoup = f.read()
            
        soup = TexSoup(stringSoup, skip_envs=skip_envs)
        soup = self.includeFiles(soup)
        soup = Converter.removeComments(soup)

        # Récupération des informations importantes
        self.stringSoup = str(soup)
        self.packages = Converter.getAllImportedPackages(soup)

        # Supprime les images précédentes
        Converter.removeOldImages(self.imgdir)
        os.makedirs(self.imgdir, exist_ok=True)

        # Récupération de la partie importante du fichier
        document = soup.find('document')
        documentList = list(document.contents)

        # Transformation en markdown
        outputString = ""

        # Ajout du titre, author et date
        outputString += "# " + Converter.removeWeirdSpacing(str(soup.find("title").string)) + "\n\n"
        outputString += "## " + Converter.removeWeirdSpacing(str(soup.find("author").string)) + "\n\n"

        outputString += self.transformSoupToMarkdown(documentList)

        # Supprime les faux espaces utilisés pour l'indentation
        outputString = outputString.replace(' ', '')

        # Ecriture dans le fichier
        with open(self.workingdir + self.filename +'.md', 'w') as f:
            f.write(outputString)

        print("Conversion terminée !")
        
if __name__ == '__main__':
    filename = "graphes (copia).tex"
    converter = Converter(filename)
    converter.run()



#!/usr/bin/env python3
"""
pnglatex, a small program that converts latex snippets to png
Copyright (C) 2017 Peijun Ma

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.

This program can be run directly as a script. It also provides the pnglatex
function which you can import and use in your own programs.
"""
from sys import stderr
from os import devnull
from subprocess import Popen, PIPE
from contextlib import contextmanager
from string import digits, ascii_letters
from pathlib import Path
from shutil import which
from argparse import ArgumentParser
from random import choice


__all__ = ['pnglatex']


_BINARIES = ('pdflatex', 'pdfcrop', 'pdf2ppm', 'pnm2png')
_TEX_BP = """\\documentclass{{article}}
\\thispagestyle{{empty}}
{packages}
\\begin{{document}}
{document}
\\end{{document}}"""


def _get_bin(name):
    """
    Get the executable name of a program based on different OS.
    """
    res = which(name)
    if '2' in name and not res:
        res = which(name.replace('2', 'to'))
    if not res:
        raise ValueError('Eexecutable {} not found'.format(name))
    return res


def _get_fname():
    """
    Get a random file name that does not exist in the current directory.
    """
    name = ''.join(choice(digits + ascii_letters) for _ in range(6))
    for f in Path('.').iterdir():
        if name in str(f):
            return _get_fname()
    return name


@contextmanager
def _cleanup(jobname):
    """
    Clean up the LaTeX mess after we compiled the tex string.
    """
    yield

    def _cleanup_suffix(suffix):
        with Path(jobname + suffix) as p:
            try:
                p.unlink()
            except FileNotFoundError:
                pass

    for suf in ('.pdf', '.out', '.aux', '.log', '-crop.pdf'):
        _cleanup_suffix(suf)


def _run(tex_string, jobname, output, null, binaries):
    """
    Run the tex string through some processes to produce a png at output.

    Returns the exit status of pnm2png
    """
    pdflatex, pdfcrop, pdf2ppm, pnm2png = binaries

    def popen(*args, stdin=None, out=null, err=null):
        return Popen(args, stdin=stdin, stdout=out, stderr=err)

    with popen(pdflatex, '-jobname=' + jobname, stdin=PIPE) as pdflatex_p:
        pdflatex_p.communicate(input=tex_string.encode(encoding='UTF-8'))

    with popen(pdfcrop, jobname + '.pdf') as crop:
        crop.wait()

    with open(output, 'wb+') as f,\
        popen(pdf2ppm, jobname + '-crop.pdf', out=PIPE) as ppm,\
            popen(pnm2png, stdin=ppm.stdout, out=f) as png:
        png.wait()
        return png.poll()


def pnglatex(tex_string, output=None, packages=[]):
    """
    Produce an png based on a input LaTeX snippet.

    @param tex_string: The LaTeX string.
    @param output: The output filename. It can also be a pathlib.Path object.
                   If not provided, this will be randomly generated.

    @return: A Path object of the output file
    @raises ValueError: If the input is empty of something went wrong with
                        the image creation.
    """
    if not tex_string:
        raise ValueError("LaTeX expression cannot be empty!")
    jobname = _get_fname()
    output = output or jobname + '.png'
    packagesString = '\n'.join(packages)
    tex_string = _TEX_BP.format(document=tex_string, packages=packagesString)
    binaries = tuple(_get_bin(b) for b in _BINARIES)

    with _cleanup(jobname), open(devnull, 'w') as null:
        status = _run(tex_string, jobname, output, null, binaries)

    if status != 0:
        with Path(output) as o:
            try:
                o.unlink()
            except FileNotFoundError:
                pass
        raise ValueError("Failed to generate png file.")
    return Path(output)


def main():
    """
    Program entry point when ran as a script.
    """
    des = 'pnglatex, a small program that converts latex snippets to png'
    parser = ArgumentParser(description=des)
    parser.add_argument('-c', help='The LaTeX string to convert',
                        required=True, metavar='"LaTeX string"')
    parser.add_argument('-o', help='The output filename.', metavar='filename')
    args = parser.parse_args()
    try:
        out = pnglatex(args.c, args.o)
    except ValueError as e:
        print(e, file=stderr)
        exit(1)
    else:
        print('Success! Your file has been saved at {}'.format(out))


if __name__ == '__main__':
    main()



"""Tokenization for all input.

Translates string into iterable `TexSoup.utils.Buffer`, yielding one
token at a time.
"""

from .utils import to_buffer, Buffer, Token, CC
from .data import arg_type
from .category import categorize  # used for tests
from .utils import IntEnum, TC
import itertools
import string

# Custom higher-level combinations of primitives
SKIP_ENV_NAMES = ('lstlisting', 'verbatim', 'verbatimtab', 'Verbatim', 'listing')
MATH_ENV_NAMES = (
    'align', 'align*', 'alignat', 'array', 'displaymath', 'eqnarray',
    'eqnarray*', 'equation', 'equation*', 'flalign', 'flalign*', 'gather',
    'gather*', 'math', 'multline', 'multline*', 'split'
)
BRACKETS_DELIMITERS = {
    '(', ')', '<', '>', '[', ']', '{', '}', r'\{', r'\}', '.' '|', r'\langle',
    r'\rangle', r'\lfloor', r'\rfloor', r'\lceil', r'\rceil', r'\ulcorner',
    r'\urcorner', r'\lbrack', r'\rbrack'
}
# TODO: looks like left-right do have to match
SIZE_PREFIX = ('left', 'right', 'big', 'Big', 'bigg', 'Bigg')
PUNCTUATION_COMMANDS = {command + bracket
                        for command in SIZE_PREFIX
                        for bracket in BRACKETS_DELIMITERS.union({'|', '.'})}

__all__ = ['tokenize']


def next_token(text, prev=None):
    r"""Returns the next possible token, advancing the iterator to the next
    position to start processing from.

    :param Union[str,iterator,Buffer] text: LaTeX to process
    :return str: the token

    >>> b = categorize(r'\textbf{Do play\textit{nice}.}   $$\min_w \|w\|_2^2$$')
    >>> print(next_token(b), next_token(b), next_token(b), next_token(b))
    \ textbf { Do play
    >>> print(next_token(b), next_token(b), next_token(b), next_token(b))
    \ textit { nice
    >>> print(next_token(b))
    }
    >>> print(next_token(categorize('.}')))
    .
    >>> next_token(b)
    '.'
    >>> next_token(b)
    '}'
    """
    while text.hasNext():
        for name, f in tokenizers:
            current_token = f(text, prev=prev)
            if current_token is not None:
                return current_token


@to_buffer()
def tokenize(text):
    r"""Generator for LaTeX tokens on text, ignoring comments.

    :param Union[str,iterator,Buffer] text: LaTeX to process

    >>> print(*tokenize(categorize(r'\\%}')))
    \\ %}
    >>> print(*tokenize(categorize(r'\textbf{hello \\%}')))
    \ textbf { hello  \\ %}
    >>> print(*tokenize(categorize(r'\textbf{Do play \textit{nice}.}')))
    \ textbf { Do play  \ textit { nice } . }
    >>> print(*tokenize(categorize(r'\begin{tabular} 0 & 1 \\ 2 & 0 \end{tabular}')))
    \ begin { tabular }  0 & 1  \\  2 & 0  \ end { tabular }
    """
    current_token = next_token(text)
    while current_token is not None:
        assert current_token.category in TC
        yield current_token
        current_token = next_token(text, prev=current_token)


##############
# Tokenizers #
##############

tokenizers = []


def token(name):
    """Marker for a token.

    :param str name: Name of tokenizer
    """

    def wrap(f):
        tokenizers.append((name, f))
        return f

    return wrap


@token('escaped_symbols')
def tokenize_escaped_symbols(text, prev=None):
    r"""Process an escaped symbol or a known punctuation command.

    :param Buffer text: iterator over line, with current position

    >>> tokenize_escaped_symbols(categorize(r'\\'))
    '\\\\'
    >>> tokenize_escaped_symbols(categorize(r'\\%'))
    '\\\\'
    >>> tokenize_escaped_symbols(categorize(r'\}'))
    '\\}'
    >>> tokenize_escaped_symbols(categorize(r'\%'))
    '\\%'
    >>> tokenize_escaped_symbols(categorize(r'\ '))
    '\\ '
    """
    if text.peek().category == CC.Escape \
            and text.peek(1) \
            and text.peek(1).category in (
                CC.Escape, CC.GroupBegin, CC.GroupEnd, CC.MathSwitch,
                CC.Alignment, CC.EndOfLine, CC.Macro, CC.Superscript,
                CC.Subscript, CC.Spacer, CC.Active, CC.Comment, CC.Other):
        result = text.forward(2)
        result.category = TC.EscapedComment
        return result


@token('comment')
def tokenize_line_comment(text, prev=None):
    r"""Process a line comment

    :param Buffer text: iterator over line, with current position

    >>> tokenize_line_comment(categorize('%hello world\\'))
    '%hello world\\'
    >>> tokenize_line_comment(categorize('hello %world'))
    >>> tokenize_line_comment(categorize('%}hello world'))
    '%}hello world'
    >>> tokenize_line_comment(categorize('%}  '))
    '%}  '
    >>> tokenize_line_comment(categorize('%hello\n world'))
    '%hello'
    >>> b = categorize(r'\\%')
    >>> _ = next(b), next(b)
    >>> tokenize_line_comment(b)
    '%'
    >>> tokenize_line_comment(categorize(r'\%'))
    """
    result = Token('', text.position)
    if text.peek().category == CC.Comment and (
            prev is None or prev.category != CC.Comment):
        result += text.forward(1)
        while text.hasNext() and text.peek().category != CC.EndOfLine:
            result += text.forward(1)
        result.category = TC.Comment
        return result


@token('math_sym_switch')
def tokenize_math_sym_switch(text, prev=None):
    r"""Group characters in math switches.

    :param Buffer text: iterator over line, with current position

    >>> tokenize_math_sym_switch(categorize(r'$\min_x$ \command'))
    '$'
    >>> tokenize_math_sym_switch(categorize(r'$$\min_x$$ \command'))
    '$$'
    """
    if text.peek().category == CC.MathSwitch:
        if text.peek(1) and text.peek(1).category == CC.MathSwitch:
            result = Token(text.forward(2), text.position)
            result.category = TC.DisplayMathSwitch
        else:
            result = Token(text.forward(1), text.position)
            result.category = TC.MathSwitch
        return result


@token('math_asym_switch')
def tokenize_math_asym_switch(text, prev=None):
    r"""Group characters in begin-end-style math switches

    :param Buffer text: iterator over line, with current position

    >>> tokenize_math_asym_switch(categorize(r'\[asf'))
    '\\['
    >>> tokenize_math_asym_switch(categorize(r'\] sdf'))
    '\\]'
    >>> tokenize_math_asym_switch(categorize(r'[]'))
    """
    mapping = {
        (CC.Escape, CC.BracketBegin):   TC.DisplayMathGroupBegin,
        (CC.Escape, CC.BracketEnd):     TC.DisplayMathGroupEnd,
        (CC.Escape, CC.ParenBegin):     TC.MathGroupBegin,
        (CC.Escape, CC.ParenEnd):       TC.MathGroupEnd
    }
    if not text.hasNext(2):
        return
    key = (text.peek().category, text.peek(1).category)
    if key in mapping:
        result = text.forward(2)
        result.category = mapping[key]
        return result


@token('line_break')
def tokenize_line_break(text, prev=None):
    r"""Extract LaTeX line breaks.

    >>> tokenize_line_break(categorize(r'\\aaa'))
    '\\\\'
    >>> tokenize_line_break(categorize(r'\aaa'))
    """
    if text.peek().category == CC.Escape and text.peek(1) \
            and text.peek(1).category == CC.Escape:
        result = text.forward(2)
        result.category = TC.LineBreak
        return result


@token('ignore')
def tokenize_ignore(text, prev=None):
    r"""Filter out ignored or invalid characters

    >>> print(*tokenize(categorize('\x00hello')))
    hello
    """
    while text.peek().category in (CC.Ignored, CC.Invalid):
        text.forward(1)


@token('spacers')
def tokenize_spacers(text, prev=None):
    r"""Combine spacers [ + line break [ + spacer]]

    >>> tokenize_spacers(categorize('\t\n{there'))
    '\t\n'
    >>> tokenize_spacers(categorize('\t\nthere'))
    >>> tokenize_spacers(categorize('      \t     '))
    '      \t     '
    >>> tokenize_spacers(categorize(r' ccc'))
    """
    result = Token('', text.position)
    while text.hasNext() and text.peek().category == CC.Spacer:
        result += text.forward(1)
    if text.hasNext() and text.peek().category == CC.EndOfLine:
        result += text.forward(1)
    while text.hasNext() and text.peek().category == CC.Spacer:
        result += text.forward(1)
    result.category = TC.MergedSpacer

    if text.hasNext() and text.peek().category in (CC.Letter, CC.Other):
        text.backward(text.position - result.position)
        return

    if result:
        return result


@token('symbols')
def tokenize_symbols(text, prev=None):
    r"""Process singletone symbols as standalone tokens.

    :param Buffer text: iterator over line, with current position. Escape is
                        isolated if not part of escaped char

    >>> next(tokenize(categorize(r'\begin turing')))
    '\\'
    >>> next(tokenize(categorize(r'\bf  {turing}')))
    '\\'
    >>> next(tokenize(categorize(r'{]}'))).category
    <TokenCode.GroupBegin: 23>
    """
    mapping = {
        CC.Escape:          TC.Escape,
        CC.GroupBegin:      TC.GroupBegin,
        CC.GroupEnd:        TC.GroupEnd,
        CC.BracketBegin:     TC.BracketBegin,
        CC.BracketEnd:    TC.BracketEnd
    }
    if text.peek().category in mapping.keys():
        result = text.forward(1)
        result.category = mapping[result.category]
        return result


# TODO: move me to parser (should parse punctuation as arg +
# store punctuation commads as macro)
@token('punctuation_command_name')
def tokenize_punctuation_command_name(text, prev=None):
    """Process command that augments or modifies punctuation.

    This is important to the tokenization of a string, as opening or closing
    punctuation is not supposed to match.

    :param Buffer text: iterator over text, with current position
    """
    if text.peek(-1) and text.peek(-1).category == CC.Escape:
        for point in PUNCTUATION_COMMANDS:
            if text.peek((0, len(point))) == point:
                result = text.forward(len(point))
                result.category = TC.PunctuationCommandName
                return result


@token('command_name')
def tokenize_command_name(text, prev=None):
    r"""Extract most restrictive subset possibility for command name.

    Parser can later join allowed spacers and macros to assemble the final
    command name and arguments.

    >>> b = categorize(r'\bf{')
    >>> _ = next(b)
    >>> tokenize_command_name(b)
    'bf'
    >>> b = categorize(r'\bf,')
    >>> _ = next(b)
    >>> tokenize_command_name(b)
    'bf'
    >>> b = categorize(r'\bf*{')
    >>> _ = next(b)
    >>> tokenize_command_name(b)
    'bf*'
    """
    if text.peek(-1) and text.peek(-1).category == CC.Escape \
            and text.peek().category == CC.Letter:
        c = text.forward(1)
        while text.hasNext() and text.peek().category == CC.Letter \
                or text.peek() == '*':  # TODO: what do about asterisk?
            # TODO: excluded other, macro, super, sub, acttive, alignment
            # although macros can make these a part of the command name
            c += text.forward(1)
        c.category = TC.CommandName
        return c


@token('string')
def tokenize_string(text, prev=None):
    r"""Process a string of text

    :param Buffer text: iterator over line, with current position
    :param Union[None,iterable,str] delimiters: defines the delimiters

    >>> tokenize_string(categorize('hello'))
    'hello'
    >>> b = categorize(r'hello again\command')
    >>> tokenize_string(b)
    'hello again'
    >>> print(b.peek())
    \
    >>> print(tokenize_string(categorize(r'0 & 1\\\command')))
    0 & 1
    """
    result = Token('', text.position, category=TC.Text)
    while text.hasNext() and text.peek().category not in (
            CC.Escape,
            CC.GroupBegin,
            CC.GroupEnd,
            CC.MathSwitch,
            CC.BracketBegin,
            CC.BracketEnd,
            CC.Comment):
        result += next(text)
    return result



from setuptools import setup, find_packages

setup(
    name="noyerTransformer",
    version="0.1.0",
    author="Elowarp",
    author_email="elowarp@gmail.com",
    description="Un convertisseur de fichier .tex en .md pour Mkdocs",
    license="MIT",
    packages=find_packages(),
    url="https://github.com/Elowarp/latexToMkdocs"
)